unit cpColorBox;
{
	v 1.0 : 22-04-1998
  Freeware component for Delphi 3 by Christian Poisson.
  Send any comments to christian.poisson@hol.fr.

  Forecolor is defined using the left button and Backcolor using the right button.
  Doubleclick opens a colordialog component (Left Doubleclick and Right Doubleclick are
  differenciated [see TcpPanel component])
  Other property : DialogOptions-->Options of the colordialog component
  New event : OnClickColorChange.
}

interface

{$R cpColorBox.dcr}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

const
  CM_CHANGECOLOR = WM_USER + 2000;
type
  TcpPanel = class(TPanel)
  protected
		fonrdblclk:TNotifyEvent;
   	procedure WMrbuttondblclk(var Message: TWMrbuttondblclk); message WM_RBUTTONDBLCLK;
  published
    property OnRDblClick:TNotifyEvent read fonrdblclk write fonrdblclk;
  end;

  TForeorBackColor = (wcFore,wcBack);
  TColorchangeevent = procedure (Sender:TObject; ForeorBackColor:TForeorBackColor; NewColor: Tcolor) of object;
  TcpColorBox = class(TCustomPanel)
  private
    { Dclarations prives }
    FColor: array [1..18] of TcpPanel;
    Fcolordialog: Tcolordialog;
    FForecolor,FBackcolor:Tcolor;
    Fcoloroption: TColorDialogOptions;
    procedure SetForeColor (value: Tcolor);
    procedure SetBackColor (value: Tcolor);
		procedure Setcoloroption (value: TColorDialogOptions);
		procedure Createcolor;
    procedure disposition;
   	procedure WMsize(var Message: TWMsize); message WM_SIZE;
    procedure siDblClick(Sender: TObject);
    procedure siRDblClick(Sender: TObject);
		procedure siMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  protected
    { Dclarations protges }
    f_on_color_change:TColorchangeevent;
 		procedure CMChangeColor(var Message: TMessage); message CM_CHANGECOLOR;
  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Dclarations publies }
    property OnClickColorChange:TColorchangeevent read f_on_color_change write f_on_color_change;
  	Property ForeColor: Tcolor read FForecolor write SetForecolor;
  	Property BackColor: Tcolor read FBackcolor write SetBackcolor;
		property DialogOptions: TColorDialogOptions read Fcoloroption write Setcoloroption default [cdFullOpen];
    property Color;
    property Showhint;
    property ParentShowhint;
    property Align;
    property BevelInner;
    property BevelOuter;
    property BorderStyle;
    property BorderWidth;
    property Ctl3D;
    property FullRepaint;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentColor;
    property ParentCtl3D;
    property PopUpMenu;
    property TabOrder;
    property TabStop;
    property Tag;
    property Visible;
    property OnResize;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure TcpPanel.WMrbuttondblclk(var Message: TWMrbuttondblclk);
begin
  if assigned(fonrdblclk) then
    fonrdblclk(self);
end;

constructor TcpColorBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  controlstyle:=controlstyle-[csAcceptsControls,csSetCaption];
  controlstyle:=controlstyle+[csFramed];
  createcolor;
  Fcolor[1].Color:=clBlack;
  Fcolor[2].Color:=clMaroon;
  Fcolor[3].Color:=clgreen;
  Fcolor[4].Color:=clOlive;
  Fcolor[5].Color:=clnavy;
  Fcolor[6].Color:=clpurple;
  Fcolor[7].Color:=clTeal;
  Fcolor[8].Color:=clgray;
  Fcolor[9].Color:=clsilver;
  Fcolor[10].Color:=clred;
  Fcolor[11].Color:=cllime;
  Fcolor[12].Color:=clYellow;
  Fcolor[13].Color:=clblue;
  Fcolor[14].Color:=clfuchsia;
  Fcolor[15].Color:=claqua;
  Fcolor[16].Color:=clwhite;
  Fcolor[17].Color:=clwhite;
  Fcolor[18].Color:=clblack;
  FForecolor:=clBlack;
  FBackcolor:=clwhite;
  width:=220;
  height:=40;
  parentctl3d:=false;
  bevelinner:=bvnone;
  bevelouter:=bvnone;
  Fcolordialog:=Tcolordialog.Create(nil);
  Fcoloroption:=[cdFullOpen];
  Fcolordialog.Options:=Fcoloroption;
end;

destructor TcpColorBox.Destroy;
begin
  inherited Destroy;
  Fcolordialog.free;
end;

procedure TcpColorBox.Createcolor;
var
	i:integer;
begin
  for i:=1 to 18 do
  	begin
    	Fcolor[i]:=TcpPanel.Create(Self);
		  Fcolor[i].Visible:=True;
		  Fcolor[i].Enabled:=True;
		  Fcolor[i].Parent:=Self;
      Fcolor[i].Bevelwidth:=2;
      Fcolor[i].Bevelinner:=bvnone;
      Fcolor[i].Bevelouter:=bvlowered;
      Fcolor[i].Bevelwidth:=2;
      Fcolor[i].controlstyle:=Fcolor[i].controlstyle-[csAcceptsControls];
      if i<17 then
      	begin
		  		Fcolor[i].OnMouseDown:=siMouseDown;
  				Fcolor[i].OnDblClick:=siDblClick;
		  		Fcolor[i].OnRDblClick:=siRDblClick;
        end;
    end;
end;

procedure TcpColorBox.CMChangeColor(var Message: TMessage);
var
	wc:TForeorBackColor;
  c:Tcolor;
begin
  if assigned(f_on_color_change) then
  	begin
    	if message.WParam=1 then
      	begin
        	wc:=wcFore;
          c:=ForeColor;
        end
      else
      	begin
	      	wc:=wcBack;
          c:=BackColor;
        end;
	    f_on_color_change(self,wc,c);
    end;
end;

procedure TcpColorBox.SetForecolor(Value: TColor);
begin
  if Value <> FForecolor then
  begin
    FForecolor:=Value;
		Fcolor[18].color:=value;
	end;
end;

procedure TcpColorBox.SetBackcolor(Value: TColor);
begin
  if Value <> FBackcolor then
  begin
    FBackcolor:=Value;
		Fcolor[17].color:=value;
	end;
end;

procedure TcpColorBox.Setcoloroption (value: TColorDialogOptions);
begin
  if Value <> Fcoloroption then
  begin
    Fcoloroption:=Value;
    Fcolordialog.Options:=value;
	end;
end;

procedure TcpColorBox.disposition;
var
	i,l,h:integer;
begin
	l:=clientwidth div 11;
  h:=clientheight div 2;
  for i:=1 to 16 do
  	begin
    	Fcolor[i].setbounds((3+(i-1) mod 8)*l,((i-1) div 8)*h,l,h);
    end;
 	Fcolor[18].setbounds(clientwidth div 20,clientheight div 10,l,h);
 	Fcolor[17].setbounds(Fcolor[18].left+Fcolor[18].width*2 div 3,Fcolor[18].top+Fcolor[18].height*2 div 3,l,h);
end;

procedure TcpColorBox.WMsize(var Message: TWMsize);
var
	w,h:integer;
begin
	w:=(clientwidth div 11)*11;
	if w<99 then w:=99;
  clientwidth:=w;
	h:=(clientheight div 2)*2;
	if h<18 then h:=18;
  clientheight:=h;
  disposition;
  inherited;
end;

procedure TcpColorBox.siDblClick(Sender: TObject);
begin
	Fcolordialog.Color:=(sender as TcpPanel).color;
  if Fcolordialog.Execute then
  	begin
    	(sender as TcpPanel).color:=Fcolordialog.color;
			Fcolor[18].Color:=Fcolordialog.color;
      if FForecolor<>Fcolor[18].color then
      	begin
					FForeColor:=Fcolor[18].color;
    		  postmessage(self.Handle,CM_CHANGECOLOR,1,0);
        end;
    end;
end;

procedure TcpColorBox.siRDblClick(Sender: TObject);
begin
	Fcolordialog.Color:=(sender as TcpPanel).color;
  if Fcolordialog.Execute then
  	begin
    	(sender as TcpPanel).color:=Fcolordialog.color;
 			Fcolor[17].Color:=Fcolordialog.color;
			if FBackcolor<>Fcolor[17].color then
      	begin
        	FBackColor:=Fcolor[17].color;
		      postmessage(self.Handle,CM_CHANGECOLOR,0,0);
        end;
    end;
end;

procedure TcpColorBox.siMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	if shift=[ssleft] then
  	begin
			Fcolor[18].Color:=(sender as TcpPanel).color;
      if FForecolor<>Fcolor[18].color then
      	begin
					FForeColor:=Fcolor[18].color;
    		  postmessage(self.Handle,CM_CHANGECOLOR,1,0);
        end;
		end
  else if shift=[ssright] then
  	begin
    	Fcolor[17].Color:=(sender as TcpPanel).color;
			if FBackcolor<>Fcolor[17].color then
      	begin
        	FBackColor:=Fcolor[17].color;
		      postmessage(self.Handle,CM_CHANGECOLOR,0,0);
        end;
    end;
end;

procedure Register;
begin
  RegisterComponents('Standard', [TcpPanel,TcpColorBox]);
end;

end.
